"""Send email notifications for all tasks that have failed."""

from orchestrator.targets import Target
from orchestrator.types import State
from orchestrator.workflow import StepList, conditional, done, init, step, workflow

from gso.services.mailer import send_mail
from gso.services.subscriptions import get_failed_tasks, get_subscription_by_process_id
from gso.settings import load_oss_params


@step("Gather all tasks that recently failed")
def gather_failed_tasks() -> State:
    """Gather all tasks that have failed."""
    return {"failed_tasks": get_failed_tasks()}


@step("Send notification emails for all failed tasks")
def send_email_notifications(state: State) -> None:
    """Send out an email notification for all tasks that have failed."""
    base_url = load_oss_params().GENERAL.public_hostname
    all_alerts = ""
    for failure in state["failed_tasks"]:
        failed_task_url = f"{base_url}/workflows/{failure["process_id"]}"
        failed_subscription = get_subscription_by_process_id(failure["process_id"])
        all_alerts = f"{all_alerts}------\n\n"
        if failed_subscription:
            all_alerts = (
                f"{all_alerts}Product name: {failed_subscription.product.name}\n"
                f"Description: {failed_subscription.description}\n"
            )
        all_alerts = (
            f'{all_alerts}The step "{failure["last_step"]}" failed for the following reason: '
            f'"{failure["failed_reason"]}".\n\nPlease inspect the full workflow at the following link: '
            f'{failed_task_url}.\n\n'
        )

    send_mail(
        "GAP - One or more tasks have failed!",
        (
            f"Please check the following tasks in GAP which have failed.\n\n{all_alerts}------"
            f"\n\nRegards, the GÉANT Automation Platform.\n\n"
        ),
    )


@workflow("Send email notifications for all failed tasks", target=Target.SYSTEM)
def task_send_email_notifications() -> StepList:
    """Gather all failed tasks, and send an email notification if needed."""
    tasks_have_failed = conditional(lambda state: len(state["failed_tasks"]) > 0)

    return init >> gather_failed_tasks >> tasks_have_failed(send_email_notifications) >> done
